//  PrintDialog.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
#include "PrintDialog.h"
#include <Translate.h>
#include <UPrint.h>
#include <UFreeType.h>

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <stdiostream.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/MessageB.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include "MYudit.h"
#include <unistd.h>


PrintDialog::PrintDialog (Widget parent, Config* configIn, UWidget* uwidgetIn)
{
	int			i;
	const char*		fName;

	topConfig=configIn;
	uWidget=uwidgetIn;
	lastButton = HELP;

	toplevel = XtVaCreatePopupShell ("PrintShell",
		xmDialogShellWidgetClass, parent,
		XmNtitle, "Yudit: Print",
		0);
	messageBox = XtVaCreateWidget ("PrintDialog",
		xmMessageBoxWidgetClass, toplevel,
		XmNdialogType, XmDIALOG_MESSAGE, 
		XmNautoUnmanage, True,
		XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		0);

	topForm = XtVaCreateManagedWidget ("PrintTopForm",
		xmFormWidgetClass, messageBox,
		0);


	frame = XtVaCreateManagedWidget ("PrintTopFrame",
		xmFrameWidgetClass, topForm,
		XmNtopAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		0);

#if ((XmVERSION>=2) || (XmVERSION==1 && XmREVISION>=2))
		XtVaCreateManagedWidget ("FrameTitle",
			xmLabelWidgetClass, frame,
			XmNchildType,	XmFRAME_TITLE_CHILD,
			XmNchildVerticalAlignment, XmALIGNMENT_CENTER,
			XtVaTypedArg, XmNlabelString,
			XmRString, Translate ("Print"), 
			strlen (Translate ("Print")) +1,
			0);
			
#endif

	titleLabel = XtVaCreateManagedWidget("TitleLabel",
		xmLabelWidgetClass, frame,
		XmNalignment, XmALIGNMENT_CENTER,
		0);

	rowColumn = XtVaCreateManagedWidget("PrintRowColumn", 
		xmRowColumnWidgetClass, topForm,
		XmNpacking,	XmPACK_COLUMN,
		XmNnumColumns, 7,
		XmNorientation, XmHORIZONTAL,
		XmNisAligned, True,
		XmNentryAlignment, XmALIGNMENT_END,
		XmNtopWidget, frame,
		XmNtopAttachment, XmATTACH_WIDGET,
		0);

	XtUnmanageChild (XmMessageBoxGetChild (messageBox,
		XmDIALOG_HELP_BUTTON));
	XtAddCallback (messageBox, XmNokCallback,
		(XtCallbackProc) MYuditPrintDialogCallback, this);
	XtAddCallback (messageBox, XmNcancelCallback,
		(XtCallbackProc) MYuditPrintDialogCallback, this);
	printer = 0;
	if (topConfig->getPrinter()!=0)
	{
		printer = new OptionMenu (rowColumn, Translate ("Printer: "));
		for (i=0; i<topConfig->getPrinterList().getSize(); i++)
		{
			printer->addItem (topConfig->getPrinterList().at(i));
		}
		printer->select ( topConfig->getPrinter());
	}
	fontName = new OptionMenu (rowColumn, Translate ("Font Name:"));
	fontCount = 0;
	for (i=0; i<topConfig->getCheckList(Config::FONTNAME).getSize()>0;i++)
	{
		fName = topConfig->getCheckList(Config::FONTNAME).at(i);
		if (UGetFreeTypePrint (fName)==0) continue;
		fontName->addItem (fName);
		fontCount++;
	}
	if (fontCount>0)
	{
		fontName->select ((const char*) topConfig->getChecked (Config::FONTNAME));
	}
	fontSize = new OptionMenu (rowColumn, Translate ("Font Size:"));
	fontSize->addList (topConfig->getCheckList(Config::FONTSIZE));
	fontSize->select ((const char*) topConfig->getChecked (Config::FONTSIZE));
	fontWeight = new OptionMenu (rowColumn, Translate ("Font Weight:"));
	fontWeight->addItem ("Medium", Translate ("Medium"));
	fontWeight->addItem ("Bold", Translate ("Bold"));
	fontWeight->select ((const char*) topConfig->getChecked (Config::FONTWEIGHT));
	fontSlant = new OptionMenu (rowColumn, Translate ("Font Slant:"));
	fontSlant->addItem ("Roman", Translate ("Roman"));
	fontSlant->addItem ("Italic", Translate ("Italic"));
	fontSlant->select ((const char*) topConfig->getChecked (Config::FONTSLANT));
	tabSize = new OptionMenu (rowColumn, Translate ("Tab Size:"));
	tabSize->addList (topConfig->getCheckList(Config::TABSIZE));
	tabSize->select ((const char*) topConfig->getChecked (Config::TABSIZE));
		
}

PrintDialog::~PrintDialog()
{
	if (tabSize!=0) delete tabSize;
	if (fontSlant!=0) delete fontSlant;
	if (fontWeight!=0) delete fontWeight;
	if (fontName!=0) delete fontName;
	if (printer!=0) delete printer;
	if (rowColumn != 0) XtDestroyWidget (rowColumn);
	if (titleLabel != 0) XtDestroyWidget (titleLabel);
	if (frame != 0) XtDestroyWidget (frame);
	if (topForm != 0) XtDestroyWidget (topForm);
	if (messageBox != 0) XtDestroyWidget (messageBox);
	if (toplevel != 0) XtDestroyWidget (toplevel);
}

PrintDialog::Button
PrintDialog::print(const UCS2* const * textIn, const int* textSizes, int textSizeIn, 
  const char* titleIn)
{
	const char*		printerName;
	const char*		printerCommand;
	const char*		title=0;
	const char*		xFontName;
	const char*		xFontSize;
	const char*		xFontWeight;
	const char*		xFontSlant;
	const char*		xTabSize;
	UFreeType*		xFreeType;
	ostream*		oStream=0;
	ofstream*		oFStream=0;
#ifdef HAVE_STDIOSTREAM
	stdiostream*		oPStream=0;
#else
	ostdiostream*		oPStream=0;
#endif
	const char*		fontFile;
	UFreeType::UWeight	weight;
	UFreeType::USlant	slant;
	UPrint*			xPrint;
	UCS2*			header;
	UPrint::UStatus		status;
	char			buffer[256];
	unsigned int		i;
	FILE*			file=0;
	int			headerSize;


	message = (const char*) 0;
	if (printer ==0 || topConfig->getPrinter()==0)
	{
		return OK;
	}
	if (fontCount==0)
	{
		message = Translate ("No True Type Fonts Defined.");
		return OK;
	}
	if (textSizeIn==0 || textIn==0 || (textSizeIn==1 && textIn[0][0] == 0))
	{
		message = Translate ("Nothing to print.");
		return OK;
	}
	title =  (titleIn==0) ? "Untitled" : titleIn;
	XtVaSetValues (titleLabel,
		XtVaTypedArg, XmNlabelString,
		XmRString, title, strlen (title) +1,
		0);
	printer->select (topConfig->getPrinter());
	XtManageChild (messageBox);
	XtPopup (toplevel, XtGrabNone);
	MWait4Confirm (messageBox);

	if (lastButton!=OK) return CANCEL;

	printerName = printer->get();
	if (printerName==0) return OK;

	printerCommand = topConfig->getPrinterCommand (printerName);
	if (printerCommand==0) return OK;

	xFontName = fontName->get();
	xFontSize = fontSize->get();
	xFontWeight = fontWeight->get();
	xFontSlant = fontSlant->get();
	xTabSize = tabSize->get();
	weight = (strcmp (xFontWeight, "Bold")==0) ? UFreeType::BOLD: UFreeType::MEDIUM;
	slant = (strcmp (xFontSlant, "Italic")==0) ? UFreeType::ITALIC: UFreeType::ROMAN;

	xFreeType = UGetFreeTypePrint (xFontName);
	if (xFreeType==0) return OK;
	
	fontFile = xFreeType->getFileName (weight, slant);
	if (fontFile==0) return OK;

	if (printerCommand[0] == '|')
	{
		file = popen (&printerCommand[1], "w");
		if (file==0)
		{
			message = "Error: can not execute '";
			message += &printerCommand[1] ;
			message +=  "'.";
			return OK;
		}
#ifdef HAVE_STDIOSTREAM
		oPStream = new stdiostream (file);
#else
		oPStream = new ostdiostream (file);
#endif
		if (oPStream==0) return OK;
		oStream = new iostream (oPStream->rdbuf());
	}
	else
	{
		oFStream =  new ofstream (printerCommand);
		if (oFStream==0) return OK;
#ifdef gnu_iostream
                if (oFStream->is_open())
#else
                if (oFStream->rdbuf() == 0 || !oFStream->rdbuf()->is_open())
#endif
		{
			message = "Can not open file: '";
			message += printerCommand;
			message += "'.";
			return OK;
		}
		oStream = oFStream;
	}

	// Finally we can start printing...
	xPrint = new UPrint (fontFile, atof (xFontSize), atoi (xTabSize));
	
	header = 0;
	headerSize = 0;
	if (titleIn !=0)
	{
		header = new UCS2[strlen (titleIn) + 1];
		for (i=0; i<strlen (titleIn); i++)
		{
			header[i] = titleIn[i];
		}
		header[i] = 0;
		headerSize = i;
	}

	status = xPrint->print (oStream, textIn, textSizes, textSizeIn, header,
		headerSize);

	if (header!=0)
	{
		delete header;
		header = 0;
	}
	

	if (status!=UPrint::OK)
	{
		message = (const char*) xPrint->errors;
	}
	else
	{
		sprintf (buffer, Translate ("Printed %d page(s)"), 
			xPrint->getPageCount());

		message = (const char*) buffer;
		if (xPrint->getWrappedCount() > 0)
		{
			message += "\n";
			sprintf (buffer, Translate ("Wrapped %d line(s)"), 
				xPrint->getWrappedCount());
			message += (const char*) buffer;
		}
	}
	delete xPrint;
	// set back default value
	topConfig->setPrinter (printerName);

	if (printerCommand[0] == '|')
	{
		delete oStream;
		delete oPStream;
		pclose (file);
	}
	else
	{
		oFStream->close();
		delete oFStream;
	}
	
	return OK;
}

const char*
PrintDialog::getMessage ()
{
	return ((const char*) message);
}

void
MYuditPrintDialogCallback (Widget w, PrintDialog* dialog, XmAnyCallbackStruct *cbs)
{
	dialog->lastButton=PrintDialog::HELP;
	if (cbs->reason == XmCR_OK)
	{
		dialog->lastButton=PrintDialog::OK;
	}
	else if (cbs->reason == XmCR_CANCEL)
	{
		dialog->lastButton=PrintDialog::CANCEL;
	}
}
